import { Authorized, Body, Delete, Get, JsonController, OnUndefined, Param, Post, QueryParam } from 'routing-controllers';
import { OpenAPI, ResponseSchema } from 'routing-controllers-openapi';
import { getConnectionManager, Repository } from 'typeorm';
import { ScanStationHasScansError, ScanStationNotFoundError } from '../errors/ScanStationErrors';
import { TrackNotFoundError } from '../errors/TrackErrors';
import { CreateScanStation } from '../models/actions/CreateScanStation';
import { ScanStation } from '../models/entities/ScanStation';
import { ResponseEmpty } from '../models/responses/ResponseEmpty';
import { ResponseScanStation } from '../models/responses/ResponseScanStation';
import { ScanController } from './ScanController';

@JsonController('/stations')
@OpenAPI({ security: [{ "AuthToken": [] }, { "RefreshTokenCookie": [] }] })
export class ScanStationController {
	private stationRepository: Repository<ScanStation>;

	/**
	 * Gets the repository of this controller's model/entity.
	 */
	constructor() {
		this.stationRepository = getConnectionManager().get().getRepository(ScanStation);
	}

	@Get()
	@Authorized("STATION:GET")
	@ResponseSchema(ResponseScanStation, { isArray: true })
	@OpenAPI({ description: 'Lists all scans (normal or track) from all runners. <br> This includes the runner\'s group and distance ran.' })
	async getAll() {
		let responseStations: ResponseScanStation[] = new Array<ResponseScanStation>();
		const stations = await this.stationRepository.find({ relations: ['track'] });
		stations.forEach(station => {
			responseStations.push(station.toResponse());
		});
		return responseStations;
	}

	@Get('/:id')
	@Authorized("STATION:GET")
	@ResponseSchema(ResponseScanStation)
	@ResponseSchema(ScanStationNotFoundError, { statusCode: 404 })
	@OnUndefined(ScanStationNotFoundError)
	@OpenAPI({ description: 'Lists all information about the runner whose id got provided.' })
	async getOne(@Param('id') id: number) {
		let scan = await this.stationRepository.findOne({ id: id }, { relations: ['track'] })
		if (!scan) { throw new ScanStationNotFoundError(); }
		return scan.toResponse();
	}

	@Post()
	@Authorized("STATION:CREATE")
	@ResponseSchema(ResponseScanStation)
	@ResponseSchema(TrackNotFoundError, { statusCode: 404 })
	@OpenAPI({ description: 'Create a new runner. <br> Please remeber to provide the runner\'s group\'s id.' })
	async post(@Body({ validate: true }) createStation: CreateScanStation) {
		let newStation = await createStation.toEntity();
		const station = await this.stationRepository.save(newStation);
		let responseStation = (await this.stationRepository.findOne({ id: station.id }, { relations: ['track'] })).toResponse();
		responseStation.key = newStation.cleartextkey;
		return responseStation;
	}

	// @Put('/:id')
	// @Authorized("SCAN:UPDATE")
	// @ResponseSchema(ResponseScan)
	// @ResponseSchema(ScanNotFoundError, { statusCode: 404 })
	// @ResponseSchema(RunnerNotFoundError, { statusCode: 404 })
	// @ResponseSchema(ScanIdsNotMatchingError, { statusCode: 406 })
	// @OpenAPI({ description: "Update the runner whose id you provided. <br> Please remember that ids can't be changed." })
	// async put(@Param('id') id: number, @Body({ validate: true }) scan: UpdateScan) {
	// 	let oldScan = await this.scanRepository.findOne({ id: id });

	// 	if (!oldScan) {
	// 		throw new ScanNotFoundError();
	// 	}

	// 	if (oldScan.id != scan.id) {
	// 		throw new ScanIdsNotMatchingError();
	// 	}

	// 	await this.scanRepository.save(await scan.updateScan(oldScan));
	// 	return (await this.scanRepository.findOne({ id: id }, { relations: ['runner'] })).toResponse();
	// }

	@Delete('/:id')
	@Authorized("STATION:DELETE")
	@ResponseSchema(ResponseScanStation)
	@ResponseSchema(ResponseEmpty, { statusCode: 204 })
	@ResponseSchema(ScanStationHasScansError, { statusCode: 406 })
	@OnUndefined(204)
	@OpenAPI({ description: 'Delete the runner whose id you provided. <br> If no runner with this id exists it will just return 204(no content).' })
	async remove(@Param("id") id: number, @QueryParam("force") force: boolean) {
		let station = await this.stationRepository.findOne({ id: id });
		if (!station) { return null; }

		const stationScans = (await this.stationRepository.findOne({ id: station.id }, { relations: ["scans"] })).scans;
		if (stationScans.length != 0 && !force) {
			throw new ScanStationHasScansError();
		}
		const scanController = new ScanController;
		for (let scan of stationScans) {
			scanController.remove(scan.id, force);
		}

		const responseStation = await this.stationRepository.findOne({ id: station.id }, { relations: ["track", "scans"] });
		await this.stationRepository.delete(station);
		return responseStation.toResponse();
	}
}
